import 'dart:io';

import 'package:android_notification_icons/config.dart';
import 'package:android_notification_icons/constants.dart';
import 'package:image/image.dart';
import 'package:meta/meta.dart';

abstract interface class IconResource {
  /// アイコンの保存ディレクトリ名
  String get dirName;

  /// 画像のサイズ
  int get size;
}

enum AndroidNotificationIconType implements IconResource {
  xxxhdpi._(dirName: 'drawable-xxxhdpi', size: 96),
  xxhdpi._(dirName: 'drawable-xxhdpi', size: 72),
  xhdpi._(dirName: 'drawable-xhdpi', size: 48),
  hdpi._(dirName: 'drawable-hdpi', size: 36),
  mdpi._(dirName: 'drawable-mdpi', size: 24);

  const AndroidNotificationIconType._(
      {required this.dirName, required this.size});

  @override
  final String dirName;
  @override
  final int size;
}

Future<void> createAndroidNotificationIcons(Config config) async {
  if (!config.validateIconName) {
    throw Exception('Invalid icon name: ${config.iconName}');
  }
  final image = await decodeImageFromFilePath(config.imagePath);
  if (!checkTransparency(image)) {
    stderr.writeln(
        'Image is not transparent: ${config.imagePath} \n You need to check the display of notification icons');
  }
  await Future.wait(
    AndroidNotificationIconType.values.map(
      (type) => createImage(
        androidNotificationIconType: type,
        iconName: config.iconName,
        image: image,
      ),
    ),
  );
}

/// アイコンの保存
@visibleForTesting
Future<void> createImage({
  required AndroidNotificationIconType androidNotificationIconType,
  required String iconName,
  required Image image,
}) async {
  final dirPath =
      '${Constants.androidResourceDir}/${androidNotificationIconType.dirName}';
  final filePath = '$dirPath/$iconName.png';
  // 画像のリサイズ
  final resizedImage = resizeImage(image, androidNotificationIconType);
  // アイコンを白色に変換
  final whiteImage = convertColoredPartsToWhite(resizedImage);
  final file = await File(filePath).create(recursive: true);
  await file.writeAsBytes(encodePng(whiteImage));
}

// ファイルパスから画像を取得
@visibleForTesting
Future<Image> decodeImageFromFilePath(String filePath) async {
  final image = decodeImage(await File(filePath).readAsBytes());
  if (image == null) {
    throw Exception('Failed to decode image: $filePath');
  }
  return image;
}

// 画像のリサイズ
@visibleForTesting
Image resizeImage(Image image, AndroidNotificationIconType type) {
  if (image.width >= type.size) {
    return copyResize(
      image,
      width: type.size,
      height: type.size,
      interpolation: Interpolation.average,
    );
  } else {
    return copyResize(
      image,
      width: image.width,
      height: image.height,
      interpolation: Interpolation.linear,
    );
  }
}

// 画像の透過チェック
@visibleForTesting
bool checkTransparency(Image image) {
  for (int x = 0; x < image.width; x++) {
    for (int y = 0; y < image.height; y++) {
      // ピクセルのアルファ値を取得
      final alpha = image.getPixel(x, y).a;
      // 一つでも透明なピクセルがあれば、画像は透過と判断
      if (alpha < 255) {
        return true;
      }
    }
  }
  // すべてのピクセルが不透明なら、画像は透過ではない
  return false;
}

// 画像を白色に変換
@visibleForTesting
Image convertColoredPartsToWhite(Image image) {
  for (int x = 0; x < image.width; x++) {
    for (int y = 0; y < image.height; y++) {
      // ピクセルの色を取得します。
      final pixel = image.getPixel(x, y);
      if (pixel.a == 0) {
        continue;
      }
      image.palette?.setRgba(pixel.index.toInt(), 255, 255, 255, 255);
    }
  }
  return image;
}
