import 'package:android_notification_icons/android_notification_icons.dart';
import 'package:image/image.dart';
import 'package:test/test.dart';

void main() {
  group('#decodeImageFromFilePath', () {
    test('画像のデコードに成功した場合、Imageを返す', () async {
      final imagePath = 'test/assets/ic_launcher.png';
      final image = await decodeImageFromFilePath(imagePath);
      expect(image, isA<Image>());
    });
    test('画像のデコードに失敗した場合、Exceptionをthrowする', () {
      final imagePath = 'test/assets/not_found.png';
      expect(
        () async => await decodeImageFromFilePath(imagePath),
        throwsA(isA<Exception>()),
      );
    });
  });
  group('#checkTransparency', () {
    test('画像が透過している場合、trueを返す', () async {
      final imagePath = 'test/assets/ic_launcher.png';
      final image = await decodeImageFromFilePath(imagePath);
      expect(checkTransparency(image), isTrue);
    });
    test('画像が透過していない場合、falseを返す', () async {
      final imagePath = 'test/assets/ic_launcher_not_transparent.png';
      final image = await decodeImageFromFilePath(imagePath);
      expect(checkTransparency(image), isFalse);
    });
  });
  group('#resizeImage', () {
    test('画像の横幅がAndroidNotificationIconTypeにて定義されているwidthより大きい場合、画像のサイズを変更する',
        () {
      final image = Image(width: 100, height: 100);
      final resizedImage =
          resizeImage(image, AndroidNotificationIconType.xxxhdpi);

      expect(
        resizedImage.width,
        equals(AndroidNotificationIconType.xxxhdpi.size),
      );
      expect(
        resizedImage.height,
        equals(AndroidNotificationIconType.xxxhdpi.size),
      );
    });

    test('画像の横幅がAndroidNotificationIconTypeにて定義されているwidthより小さい場合、サイズ変更は行わない',
        () {
      final image = Image(width: 20, height: 20);
      final resizedImage = resizeImage(image, AndroidNotificationIconType.mdpi);

      expect(resizedImage.width, equals(image.width));
      expect(resizedImage.height, equals(image.height));
    });
  });
  group('#convertColoredPartsToWhite', () {
    late Image image;
    setUp(() {
      image = Image(
        width: 10,
        height: 10,
        numChannels: 4,
        withPalette: true,
      );
    });
    test('白以外の色は白に変換', () async {
      for (final pixel in image) {
        image.palette?.setRgba(pixel.index.toInt(), 255, 0, 0, 255);
      }
      final result = convertColoredPartsToWhite(image);
      for (int x = 0; x < result.width; x++) {
        for (int y = 0; y < result.height; y++) {
          final pixel = result.getPixel(x, y);
          expect(pixel.r, equals(255));
          expect(pixel.g, equals(255));
          expect(pixel.b, equals(255));
          expect(pixel.a, equals(255));

          expect(result.palette?.getRed(pixel.index.toInt()), equals(255));
          expect(result.palette?.getGreen(pixel.index.toInt()), equals(255));
          expect(result.palette?.getBlue(pixel.index.toInt()), equals(255));
          expect(result.palette?.getAlpha(pixel.index.toInt()), equals(255));
        }
      }
    });

    test('透明部分の透過性は変更しない', () async {
      for (final pixel in image) {
        image.palette?.setRgba(pixel.index.toInt(), 0, 0, 0, 0);
      }
      final result = convertColoredPartsToWhite(image);
      for (int x = 0; x < result.width; x++) {
        for (int y = 0; y < result.height; y++) {
          final pixel = result.getPixel(x, y);
          expect(pixel.a, equals(0));
          expect(result.palette?.getAlpha(pixel.index.toInt()), equals(0));
        }
      }
    });

    test('白色は変更しない', () async {
      for (final pixel in image) {
        image.palette?.setRgba(pixel.index.toInt(), 255, 255, 255, 255);
      }
      final result = convertColoredPartsToWhite(image);
      for (int x = 0; x < result.width; x++) {
        for (int y = 0; y < result.height; y++) {
          final pixel = result.getPixel(x, y);
          expect(pixel.r, equals(255));
          expect(pixel.g, equals(255));
          expect(pixel.b, equals(255));
          expect(pixel.a, equals(255));

          expect(result.palette?.getRed(pixel.index.toInt()), equals(255));
          expect(result.palette?.getGreen(pixel.index.toInt()), equals(255));
          expect(result.palette?.getBlue(pixel.index.toInt()), equals(255));
          expect(result.palette?.getAlpha(pixel.index.toInt()), equals(255));
        }
      }
    });
  });
}
